package interpreteur.parser.lexical;

import interpreteur.parser.SyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Admin
 */
public class ParserLexical
{

	private List<String> separateurs;
	public static String FIN_INSTRUCTION = "#";
	public static String NON = "!";
	private boolean debug;

	public ParserLexical()
	{
		separateurs = new LinkedList<String>();

		// On initialise cette liste avec les séparateurs connus
		separateurs.add(new String(FIN_INSTRUCTION));
		separateurs.add("==");
		separateurs.add("!=");
		separateurs.add(">=");
		separateurs.add("<=");
		separateurs.add(">");
		separateurs.add("<");
		separateurs.add("=");
		separateurs.add("+");
		separateurs.add("-");
		separateurs.add("*");
		separateurs.add("/");
		separateurs.add("(");
		separateurs.add(")");
		separateurs.add("et");
		separateurs.add("ou");
		separateurs.add(new String(NON));

		debug = false;

	}

	public ArrayList<MotLexical> analyse(String input) throws SyntaxException
	{
		if (debug)
		{
			System.out.println("== Analyse lexicale ==");
		}
		ArrayList<MotLexical> sequence = new ArrayList<MotLexical>();
		int lastIndex = 0;

		for (int i = 0; i < input.length(); ++i)
		{
			// On regarde si le caractère i est un séparateur
			boolean search = true;
			Iterator<String> it = separateurs.iterator();
			while (search && it.hasNext())
			{
				String sep = it.next();

				// Là, on a trouvé un séparateur
				if (input.substring(i).startsWith(sep))
				{
					// Plusieurs cas à gérer en fonction du type de séparateur
					if (sep.equals(NON))
					{

						// On regarde avant nous
						// Mais, qu'est-ce qu'était le mot contenu entre ce séparateur
						// et celui d'avant ?
						if (i != lastIndex)
						{
							String s = input.substring(lastIndex, i);
							MotLexical mot = analyseMot(s);
							// On ajoute le tout à la séquence
							sequence.add(mot);
						}
						sequence.add(new MotLexical(MotLexical.TYPE_NON, sep));

						lastIndex = i + sep.length();
						search = false;
						i += sep.length() - 1;

					}
					else if (sep.equals("("))
					{
						MotLexical motSep = new MotLexical(MotLexical.TYPE_PARENTHESE_OUVRANTE, sep);
						sequence.add(motSep);
						lastIndex = i + sep.length();
						search = false;
						i += sep.length() - 1;

					}
					else
					{
						MotLexical motSep = new MotLexical(MotLexical.TYPE_OPERATEUR, sep);
						if (sep.equals(new String(FIN_INSTRUCTION)))
						{
							motSep = new MotLexical(MotLexical.TYPE_FIN_INSTRUCTION, sep);
						}
						if (sep.equals(new String("=")))
						{
							motSep = new MotLexical(MotLexical.TYPE_AFFECTATION, sep);
						}

						if (sep.equals(")"))
						{
							motSep = new MotLexical(MotLexical.TYPE_PARENTHESE_FERMANTE, sep);
						}

						// On regarde avant nous
						// Mais, qu'est-ce qu'était le mot contenu entre ce séparateur
						// et celui d'avant ?
						if (i != lastIndex)
						{
							String s = input.substring(lastIndex, i);
							MotLexical mot = analyseMot(s);
							// On ajoute le tout à la séquence
							sequence.add(mot);
						}
						// On ajoute le séparateur à la séquence
						sequence.add(motSep);
						search = false;
						lastIndex = i + sep.length();
						i += sep.length() - 1;
					}
				}
			}

		}
		
		if(debug)
		{
			System.out.println("Séquence reconnue : ");
			int i = 0;
			for(MotLexical m : sequence)
			{
				System.out.println("["+i+"] " + m.getValeur());
				i++;
			}
		}
		return sequence;
	}

	private MotLexical analyseMot(
					String mot) throws SyntaxException
	{
		int typeMot = MotLexical.TYPE_VARIABLE;

		if (mot.equals(NON))
		{
			return new MotLexical(MotLexical.TYPE_NON, mot);
		}

		if (mot.equals(FIN_INSTRUCTION))
		{
			return new MotLexical(MotLexical.TYPE_FIN_INSTRUCTION, mot);
		}

		if (mot.equals("vrai"))
		{
			return new MotLexical(MotLexical.TYPE_BOOLEEN, mot);
		}

		if (mot.equals("faux"))
		{
			return new MotLexical(MotLexical.TYPE_BOOLEEN, mot);
		}

		if (isNombre(mot))
		{
			return new MotLexical(MotLexical.TYPE_NOMBRE, mot);
		}

		if (isVariable(mot))
		{
			return new MotLexical(MotLexical.TYPE_VARIABLE, mot);
		}

		throw new SyntaxException("L'identificateur '" + mot + "' n'est pas valide.");

	}

	private boolean isNombre(String s)
	{
		s = s.replaceAll(",", ".");
		try
		{
			double nombre = Double.parseDouble(s);
		} catch (Exception e)
		{
			return false;
		}
		return true;
	}

	private boolean isVariable(String s)
	{
		/*
		 * L'expression régulière est :
		 * \\p{Alpha}(\\p{Alpha}|\\p{Digit})*
		 * 
		 * Ce qui autorise les noms commencant par une lettre
		 * non accentuée suivie de lettres et de chiffres
		 */

		Pattern p = Pattern.compile("\\p{Alpha}(\\p{Alpha}|\\p{Digit})*");
		Matcher m = p.matcher(s);
		return m.matches();
	}

	public boolean isDebug()
	{
		return debug;
	}

	public void setDebug(boolean debug)
	{
		this.debug = debug;
	}
}
